/*
 * Part of Intel(R) Manageability Engine Interface Linux driver
 *
 * Copyright (c) 2003 - 2008 Intel Corp.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions, and the following disclaimer,
 *    without modification.
 * 2. Redistributions in binary form must reproduce at minimum a disclaimer
 *    substantially similar to the "NO WARRANTY" disclaimer below
 *    ("Disclaimer") and any redistribution must be conditioned upon
 *    including a substantially similar Disclaimer requirement for further
 *    binary redistribution.
 * 3. Neither the names of the above-listed copyright holders nor the names
 *    of any contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * Alternatively, this software may be distributed under the terms of the
 * GNU General Public License ("GPL") version 2 as published by the Free
 * Software Foundation.
 *
 * NO WARRANTY
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR CONTRIBUTORS BE LIABLE FOR SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGES.
 *
 */

#include "heci.h"
#include "heci_version.h"


#define HECI_READ_TIMEOUT	45

#define HECI_DRIVER_NAME	"heci"

/*
 *  heci driver strings
 */
char heci_driver_name[] = HECI_DRIVER_NAME;
char heci_driver_string[] = "Intel(R) Management Engine Interface";
char heci_driver_version[] = HECI_DRIVER_VERSION;
char heci_copyright[] = "Copyright (c) 2003 - 2008 Intel Corporation.";


#ifdef HECI_DEBUG
int heci_debug = 1;
#else
int heci_debug;
#endif
MODULE_PARM_DESC(heci_debug,  "Debug enabled or not");
module_param(heci_debug, int, 0644);


#define HECI_DEV_NAME	"heci"

/* heci char device for registration */
static struct cdev heci_cdev;

/* major number for device */
static int heci_major;
/* The device pointer */
static struct pci_dev *heci_device;

struct class *heci_class;
struct device *heci_dev;

/* heci_pci_tbl - PCI Device ID Table */
static struct pci_device_id heci_pci_tbl[] = {
	{PCI_DEVICE(PCI_VENDOR_ID_INTEL, HECI_DEV_ID_82946GZ)},
	{PCI_DEVICE(PCI_VENDOR_ID_INTEL, HECI_DEV_ID_82G35)},
	{PCI_DEVICE(PCI_VENDOR_ID_INTEL, HECI_DEV_ID_82Q965)},
	{PCI_DEVICE(PCI_VENDOR_ID_INTEL, HECI_DEV_ID_82G965)},
	{PCI_DEVICE(PCI_VENDOR_ID_INTEL, HECI_DEV_ID_82GM965)},
	{PCI_DEVICE(PCI_VENDOR_ID_INTEL, HECI_DEV_ID_82GME965)},
	{PCI_DEVICE(PCI_VENDOR_ID_INTEL, HECI_DEV_ID_ICH9_82Q35)},
	{PCI_DEVICE(PCI_VENDOR_ID_INTEL, HECI_DEV_ID_ICH9_82G33)},
	{PCI_DEVICE(PCI_VENDOR_ID_INTEL, HECI_DEV_ID_ICH9_82Q33)},
	{PCI_DEVICE(PCI_VENDOR_ID_INTEL, HECI_DEV_ID_ICH9_82X38)},
	{PCI_DEVICE(PCI_VENDOR_ID_INTEL, HECI_DEV_ID_ICH9_3200)},
	{PCI_DEVICE(PCI_VENDOR_ID_INTEL, HECI_DEV_ID_ICH9_6)},
	{PCI_DEVICE(PCI_VENDOR_ID_INTEL, HECI_DEV_ID_ICH9_7)},
	{PCI_DEVICE(PCI_VENDOR_ID_INTEL, HECI_DEV_ID_ICH9_8)},
	{PCI_DEVICE(PCI_VENDOR_ID_INTEL, HECI_DEV_ID_ICH9_9)},
	{PCI_DEVICE(PCI_VENDOR_ID_INTEL, HECI_DEV_ID_ICH9_10)},
	{PCI_DEVICE(PCI_VENDOR_ID_INTEL, HECI_DEV_ID_ICH9M_1)},
	{PCI_DEVICE(PCI_VENDOR_ID_INTEL, HECI_DEV_ID_ICH9M_2)},
	{PCI_DEVICE(PCI_VENDOR_ID_INTEL, HECI_DEV_ID_ICH9M_3)},
	{PCI_DEVICE(PCI_VENDOR_ID_INTEL, HECI_DEV_ID_ICH9M_4)},
	{PCI_DEVICE(PCI_VENDOR_ID_INTEL, HECI_DEV_ID_ICH10_1)},
	{PCI_DEVICE(PCI_VENDOR_ID_INTEL, HECI_DEV_ID_ICH10_2)},
	{PCI_DEVICE(PCI_VENDOR_ID_INTEL, HECI_DEV_ID_ICH10_3)},
	{PCI_DEVICE(PCI_VENDOR_ID_INTEL, HECI_DEV_ID_ICH10_4)},
	{PCI_DEVICE(PCI_VENDOR_ID_INTEL, HECI_DEV_ID_ICH10_5)},
	/* required last entry */
	{0, }
};

MODULE_DEVICE_TABLE(pci, heci_pci_tbl);

/*
 * Local Function Prototypes
 */
static int __init heci_init_module(void);
static void __exit heci_exit_module(void);
static int __devinit heci_probe(struct pci_dev *pdev,
				const struct pci_device_id *ent);
static void __devexit heci_remove(struct pci_dev *pdev);
static int heci_open(struct inode *inode, struct file *file);
static int heci_release(struct inode *inode, struct file *file);
static ssize_t heci_read(struct file *file, char __user *ubuf,
			 size_t length, loff_t *offset);
static long heci_ioctl(struct file *file, unsigned int cmd, unsigned long data);
static ssize_t heci_write(struct file *file, const char __user *ubuf,
			  size_t length, loff_t *offset);
static unsigned int heci_poll(struct file *file, poll_table *wait);
#ifdef CONFIG_PM
static int heci_suspend(struct pci_dev *pdev, pm_message_t state);
static int heci_resume(struct pci_dev *pdev);
static __u16 g_sus_wd_timeout;
#else
#define heci_suspend NULL
#define heci_resume NULL
#endif
/*
 *  PCI driver structure
 */
static struct pci_driver heci_driver = {
	.name = heci_driver_name,
	.id_table = heci_pci_tbl,
	.probe = heci_probe,
	.remove = __devexit_p(heci_remove),
	.shutdown = __devexit_p(heci_remove),
	.suspend = heci_suspend,
	.resume = heci_resume
};

/*
 * file operations structure will be use heci char device.
 */
static struct file_operations heci_fops = {
	.owner = THIS_MODULE,
	.read = heci_read,
	.unlocked_ioctl = heci_ioctl,
	.open = heci_open,
	.release = heci_release,
	.write = heci_write,
	.poll = heci_poll,
};

/**
 * heci_registration_cdev - set up the cdev structure for heci device.
 *
 * @dev: char device struct
 * @hminor: minor number for registration char device
 * @fops: file operations structure
 *
 * returns 0 on success, <0 on failure.
 */
static int heci_registration_cdev(struct cdev *dev, int hminor,
				  struct file_operations *fops)
{
	int ret, devno = MKDEV(heci_major, hminor);

	cdev_init(dev, fops);
	dev->owner = THIS_MODULE;
	ret = cdev_add(dev, devno, 1);
	/* Fail gracefully if need be */
	if (ret) {
		printk(KERN_ERR "heci: Error %d registering heci device %d\n",
		       ret, hminor);
	}
	return ret;
}

/* Display the version of heci driver. */
static ssize_t version_show(struct class *dev, char *buf)
{
	return sprintf(buf, "%s %s.\n",
		       heci_driver_string, heci_driver_version);
}

static CLASS_ATTR(version, S_IRUGO, version_show, NULL);

/**
 * heci_register_cdev - registers heci char device
 *
 * returns 0 on success, <0 on failure.
 */
static int heci_register_cdev(void)
{
	int ret;
	dev_t dev;

	/* registration of char devices */
	ret = alloc_chrdev_region(&dev, HECI_MINORS_BASE, HECI_MINORS_COUNT,
				  HECI_DRIVER_NAME);
	if (ret) {
		printk(KERN_ERR "heci: Error allocating char device region.\n");
		return ret;
	}

	heci_major = MAJOR(dev);

	ret = heci_registration_cdev(&heci_cdev, HECI_MINOR_NUMBER,
				     &heci_fops);
	if (ret)
		unregister_chrdev_region(MKDEV(heci_major, HECI_MINORS_BASE),
					 HECI_MINORS_COUNT);

	return ret;
}

/**
 * heci_unregister_cdev - unregisters heci char device
 */
static void heci_unregister_cdev(void)
{
	cdev_del(&heci_cdev);
	unregister_chrdev_region(MKDEV(heci_major, HECI_MINORS_BASE),
				 HECI_MINORS_COUNT);
}

#ifndef HECI_DEVICE_CREATE
#define HECI_DEVICE_CREATE device_create
#endif
#ifndef HECI_STRUCT_DEVICE_PTR
#define HECI_STRUCT_DEVICE_PTR(ptr) ptr
#endif
/**
 * heci_sysfs_device_create - adds device entry to sysfs
 *
 * returns 0 on success, <0 on failure.
 */
static int heci_sysfs_device_create(void)
{
	struct class *class;
	void *tmphdev;
	int err = 0;

	class = class_create(THIS_MODULE, HECI_DRIVER_NAME);
	if (IS_ERR(class)) {
		err = PTR_ERR(class);
		printk(KERN_ERR "heci: Error creating heci class.\n");
		goto err_out;
	}

	err = class_create_file(class, &class_attr_version);
	if (err) {
		class_destroy(class);
		printk(KERN_ERR "heci: Error creating heci class file.\n");
		goto err_out;
	}

	tmphdev = HECI_DEVICE_CREATE(class, NULL, heci_cdev.dev, NULL,
					HECI_DEV_NAME);
	if (IS_ERR(tmphdev)) {
		err = PTR_ERR(tmphdev);
		class_remove_file(class, &class_attr_version);
		class_destroy(class);
		goto err_out;
	}

	heci_dev = HECI_STRUCT_DEVICE_PTR(tmphdev);
	heci_class = class;
err_out:
	return err;
}

/**
 * heci_sysfs_device_remove - unregisters the device entry on sysfs
 */
static void heci_sysfs_device_remove(void)
{
	if ((heci_class == NULL) || (IS_ERR(heci_class)))
		return;

	heci_dev = NULL;
	device_destroy(heci_class, heci_cdev.dev);
	class_remove_file(heci_class, &class_attr_version);
	class_destroy(heci_class);
}

/**
 * heci_init_module - Driver Registration Routine
 *
 * heci_init_module is the first routine called when the driver is
 * loaded. All it does is register with the PCI subsystem.
 *
 * returns 0 on success, <0 on failure.
 */
static int __init heci_init_module(void)
{
	int ret = 0;

	printk(KERN_INFO "heci: %s - version %s\n", heci_driver_string,
			heci_driver_version);
	printk(KERN_INFO "heci: %s\n", heci_copyright);

	/* init pci module */
	ret = pci_register_driver(&heci_driver);
	if (ret < 0) {
		printk(KERN_ERR "heci: Error registering driver.\n");
		goto end;
	}

	ret = heci_register_cdev();
	if (ret)
		goto unregister_pci;

	ret = heci_sysfs_device_create();
	if (ret)
		goto unregister_cdev;

	return ret;

unregister_cdev:
	heci_unregister_cdev();
unregister_pci:
	pci_unregister_driver(&heci_driver);
end:
	return ret;
}

module_init(heci_init_module);


/**
 * heci_exit_module - Driver Exit Cleanup Routine
 *
 * heci_exit_module is called just before the driver is removed
 * from memory.
 */
static void __exit heci_exit_module(void)
{
	pci_unregister_driver(&heci_driver);
	heci_sysfs_device_remove();
	heci_unregister_cdev();
}

module_exit(heci_exit_module);


/**
 * heci_probe - Device Initialization Routine
 *
 * @pdev: PCI device information struct
 * @ent: entry in kcs_pci_tbl
 *
 * returns 0 on success, <0 on failure.
 */
static int __devinit heci_probe(struct pci_dev *pdev,
				const struct pci_device_id *ent)
{
	struct iamt_heci_device *dev = NULL;
	int i, err = 0;

	if (heci_device) {
		err = -EEXIST;
		goto end;
	}
	/* enable pci dev */
	err = pci_enable_device(pdev);
	if (err) {
		printk(KERN_ERR "heci: Failed to enable pci device.\n");
		goto end;
	}
	/* set PCI host mastering  */
	pci_set_master(pdev);
	/* pci request regions for heci driver */
	err = pci_request_regions(pdev, heci_driver_name);
	if (err) {
		printk(KERN_ERR "heci: Failed to get pci regions.\n");
		goto disable_device;
	}
	/* allocates and initializes the heci dev structure */
	dev = init_heci_device(pdev);
	if (dev == NULL) {
		err = -ENOMEM;
		goto release_regions;
	}
	/* mapping  IO device memory */
	for (i = 0; i <= 5; i++) {
		if (pci_resource_len(pdev, i) == 0)
			continue;
		if (pci_resource_flags(pdev, i) & IORESOURCE_IO) {
			printk(KERN_ERR "heci: heci has IO ports.\n");
			goto free_device;
		} else if (pci_resource_flags(pdev, i) & IORESOURCE_MEM) {
			if (dev->mem_base) {
				printk(KERN_ERR
					"heci: Too many mem addresses.\n");
				goto free_device;
			}
			dev->mem_base = pci_resource_start(pdev, i);
			dev->mem_length = pci_resource_len(pdev, i);
		}
	}
	if (!dev->mem_base) {
		printk(KERN_ERR "heci: No address to use.\n");
		err = -ENODEV;
		goto free_device;
	}
	dev->mem_addr = ioremap_nocache(dev->mem_base,
			dev->mem_length);
	if (dev->mem_addr == NULL) {
		printk(KERN_ERR "heci: Remap IO device memory failure.\n");
		err = -ENOMEM;
		goto free_device;
	}

	dev->isr_thread = kthread_run(heci_isr_thread, dev, "heci_isr_thread");
	if (IS_ERR(dev->isr_thread)) {
		int rc = PTR_ERR(dev->isr_thread);
		printk(KERN_ERR "heci: Unable to"
			"start the heci isr thread: %d\n", rc);
		dev->isr_thread = NULL;
		goto free_device;
	}

	/* request and enable interrupt   */
	err = request_irq(pdev->irq, heci_isr_interrupt, IRQF_SHARED,
			heci_driver_name, dev);
	if (err) {
		printk(KERN_ERR "heci: Request_irq failure. irq = %d \n",
		       pdev->irq);
		goto stop_isr;
	}

	init_timer(&dev->wd_timer);

	if (heci_hw_init(dev)) {
		printk(KERN_ERR "heci: Init hw failure.\n");
		err = -ENODEV;
		goto release_irq;
	}

	heci_initialize_clients(dev);
	if (atomic_read(&dev->heci_state) != HECI_ENABLED) {
		err = -ENODEV;
		goto release_hw;
	}

	spin_lock_bh(&dev->device_lock);
	heci_device = pdev;
	pci_set_drvdata(pdev, dev);
	spin_unlock_bh(&dev->device_lock);

	if (dev->wd_timeout)
		mod_timer(&dev->wd_timer, jiffies);

#ifdef CONFIG_PM
	g_sus_wd_timeout = 0;
#endif
	printk(KERN_INFO "heci driver initialization successful.\n");
	return 0;

release_hw:
	/* disable interrupts */
	spin_lock_bh(&dev->hw_lock);
	dev->host_hw_state = read_heci_register(dev, H_CSR);
	heci_csr_disable_interrupts(dev);
	spin_unlock_bh(&dev->hw_lock);

	flush_scheduled_work();
release_irq:
	del_timer_sync(&dev->wd_timer);
	free_irq(pdev->irq, dev);
stop_isr:
	spin_lock_bh(&dev->device_lock);
	atomic_set(&dev->heci_state, HECI_REMOVING);
	if (dev->isr_thread != NULL) {
		set_tsk_thread_flag(dev->isr_thread, TIF_SIGPENDING);
		wake_up_process(dev->isr_thread);
		dev->isr_thread = NULL;
	}
	spin_unlock_bh(&dev->device_lock);
free_device:
	free_heci_device(dev);
release_regions:
	pci_release_regions(pdev);
disable_device:
	pci_disable_device(pdev);
end:
	printk(KERN_ERR "heci driver initialization failed.\n");
	return err;
}


static void _heci_stop_wd(struct iamt_heci_device *dev)
{
	del_timer_sync(&dev->wd_timer);

	if (atomic_read(&dev->wd_conn.state) != HECI_FILE_CONNECTED)
		return;

	spin_lock_bh(&dev->device_lock);
	spin_lock(&dev->wd_conn.file_lock);
#ifdef CONFIG_PM
	g_sus_wd_timeout = dev->wd_timeout;
#endif
	if (dev->wd_timeout == 0) {
		spin_unlock(&dev->wd_conn.file_lock);
		spin_unlock_bh(&dev->device_lock);
		return;
	}
	dev->wd_timeout = 0;
	dev->wd_due_counter = 0;
	memcpy(dev->wd_data, heci_stop_wd_params, HECI_WD_PARAMS_SIZE);
	dev->wd_stop = 1;
	atomic_set(&dev->wd_stoped, 0);
	if (heci_wd(dev, NULL) != 0) {
		atomic_inc(&dev->recvd_wrr);
		wake_up_interruptible(&dev->wait_isr);
	}
	spin_unlock(&dev->wd_conn.file_lock);
	spin_unlock_bh(&dev->device_lock);

	atomic_set(&dev->wd_conn.state, HECI_FILE_DISCONNECTING);
	heci_disconnect_host_client(dev, &dev->wd_conn);

	wait_event_interruptible_timeout(dev->wait_stop_wd,
			(atomic_read(&dev->wd_stoped) != 0), 10 * HZ);
	if (atomic_read(&dev->wd_stoped) == 0)
		DBG("stop wd failed to complete.\n");
	else
		DBG("stop wd complete.\n");
}

static int _heci_file_cancel_list(struct heci_io_list *io_list,
		struct file *file)
{
	struct heci_cb_private *priv_cb_pos = NULL;
	struct heci_cb_private *priv_cb_next = NULL;
	int rets = 0;

	list_for_each_entry_safe(priv_cb_pos, priv_cb_next,
				 &io_list->cb_list, cb_list) {
		if (priv_cb_pos->file_object == file) {
			list_del_init(&priv_cb_pos->cb_list);
			heci_free_io_cb(priv_cb_pos);
			rets = 1;
		}
	}
	return rets;
}

static int _heci_file_cancel(struct iamt_heci_device *dev, struct file *file,
				struct heci_connection *conn)
{
	spin_lock_bh(&dev->cmdl_lock);
	if (conn == &dev->iamthif_conn)
		_heci_file_cancel_list(&dev->pthi_cmd_list, file);
	else
		_heci_file_cancel_list(&dev->heci_cmd_list, file);
	spin_unlock_bh(&dev->cmdl_lock);

	spin_lock_bh(&dev->crrdl_lock);
	_heci_file_cancel_list(&dev->read_list, file);
	spin_unlock_bh(&dev->crrdl_lock);

	spin_lock_bh(&dev->rdcpl_lock);
	_heci_file_cancel_list(&dev->read_cmpl_list, file);
	spin_unlock_bh(&dev->rdcpl_lock);

	return 0;
}

static void _heci_cancel_list(struct heci_io_list *io_list)
{
	struct heci_cb_private *priv_cb_pos = NULL;
	struct heci_cb_private *priv_cb_next = NULL;

	list_for_each_entry_safe(priv_cb_pos, priv_cb_next,
				 &io_list->cb_list, cb_list) {
		list_del_init(&priv_cb_pos->cb_list);
		heci_free_io_cb(priv_cb_pos);
	}
}

static void _heci_cancel_all_lists(struct iamt_heci_device *dev)
{
	spin_lock(&dev->cmdl_lock);
	_heci_cancel_list(&dev->pthi_cmd_list);
	_heci_cancel_list(&dev->heci_cmd_list);
	spin_unlock(&dev->cmdl_lock);

	spin_lock(&dev->crrdl_lock);
	_heci_cancel_list(&dev->ctrl_wr_list);
	_heci_cancel_list(&dev->ctrl_rd_list);
	_heci_cancel_list(&dev->read_list);
	spin_unlock(&dev->crrdl_lock);

	spin_lock(&dev->rdcpl_lock);
	_heci_cancel_list(&dev->read_cmpl_list);
	spin_unlock(&dev->rdcpl_lock);

	_heci_cancel_list(&dev->write_list);
	_heci_cancel_list(&dev->cmpl_list);
}

/**
 * heci_remove - Device Removal Routine
 *
 * @pdev: PCI device information struct
 *
 * heci_remove is called by the PCI subsystem to alert the driver
 * that it should release a PCI device.
 */
static void __devexit heci_remove(struct pci_dev *pdev)
{
	struct iamt_heci_device *dev = pci_get_drvdata(pdev);

	if (dev == NULL)
		return;

	if (heci_device != pdev)
		return;

	spin_lock_bh(&dev->device_lock);
	if (heci_device != pdev) {
		spin_unlock_bh(&dev->device_lock);
		return;
	}
	heci_device = NULL;
	if (dev->reinit_tsk != NULL) {
		set_tsk_thread_flag(dev->reinit_tsk, TIF_SIGPENDING);
		wake_up_process(dev->reinit_tsk);
	}
	spin_unlock_bh(&dev->device_lock);
	down_trylock(&dev->reinit_sem); /* lock if can */

	if (atomic_cmpxchg(&dev->iamthif_conn.state, HECI_FILE_CONNECTED,
			    HECI_FILE_DISCONNECTING) == HECI_FILE_CONNECTED)
		heci_disconnect_host_client(dev, &dev->iamthif_conn);

	_heci_stop_wd(dev);

	atomic_set(&dev->heci_state, HECI_REMOVING);

	spin_lock_bh(&dev->device_lock);
	/* disable interrupts */
	spin_lock(&dev->hw_lock);
	heci_csr_disable_interrupts(dev);
	spin_unlock(&dev->hw_lock);
	/* remove entry if already in list */
	spin_lock(&dev->conn_lock);
	DBG("list del iamthif and wd conn list.\n");
	list_del_init(&dev->wd_conn.link);
	list_del_init(&dev->iamthif_conn.link);
	spin_unlock(&dev->conn_lock);

	spin_lock(&dev->iamthif_conn.file_lock);
	dev->iamthif_conn.read_cb = NULL;
	dev->iamthif_conn.file = NULL;
	spin_unlock(&dev->iamthif_conn.file_lock);

	_heci_cancel_all_lists(dev);

	spin_lock(&dev->mecl_lock);
	dev->num_heci_me_clients = 0;
	spin_unlock(&dev->mecl_lock);
	spin_unlock_bh(&dev->device_lock);

	flush_scheduled_work();

	free_irq(pdev->irq, dev);

	spin_lock_bh(&dev->device_lock);
	if (dev->isr_thread != NULL) {
		set_tsk_thread_flag(dev->isr_thread, TIF_SIGPENDING);
		wake_up_process(dev->isr_thread);
		dev->isr_thread = NULL;
	}
	spin_unlock_bh(&dev->device_lock);

	pci_set_drvdata(pdev, NULL);
	free_heci_device(dev);
	pci_release_regions(pdev);
	pci_disable_device(pdev);
}


/**
 * heci_open - the open function
 *
 * @inode: pointer to inode structure
 * @file: pointer to file structure
 *
 * returns 0 on success, <0 on error
 */
static int heci_open(struct inode *inode, struct file *file)
{
	struct heci_file_priv *file_ext = NULL;
	struct heci_connection *conn = NULL;
	struct iamt_heci_device *dev;

	DBG("entry\n");
	if (heci_device == NULL)
		return -ENODEV;

	if (iminor(inode) != HECI_MINOR_NUMBER)
		return -ENODEV;

	dev = pci_get_drvdata(heci_device);
	if (dev == NULL)
		return -ENODEV;

	file_ext = heci_alloc_file_priv(file);
	if (file_ext == NULL)
		return -ENOMEM;

	conn = heci_alloc_heci_connection(file);
	if (conn == NULL) {
		kfree(file_ext);
		return -ENOMEM;
	}

	if (atomic_read(&dev->heci_state) != HECI_ENABLED) {
		kfree(conn);
		kfree(file_ext);
		return -ENODEV;
	}
	spin_lock_bh(&dev->conn_lock);
	if (dev->open_handle_count >= HECI_MAX_OPEN_HANDLE_COUNT) {
		spin_unlock_bh(&dev->conn_lock);
		kfree(conn);
		kfree(file_ext);
		return -ENFILE;
	}
	dev->open_handle_count++;

	do {
		dev->current_host_client_id++; /* allow overflow */
	} while ((dev->heci_host_clients[dev->current_host_client_id / 8]
		& (1 << (dev->current_host_client_id % 8))) != 0);

	DBG("current_host_client_id = %d\n", dev->current_host_client_id);
	DBG("dev->open_handle_count = %lu\n", dev->open_handle_count);

	spin_lock(&conn->file_lock);
	conn->host_client_id = dev->current_host_client_id;
	file_ext->host_client_id = conn->host_client_id;
	dev->heci_host_clients[conn->host_client_id / 8] |=
		(1 << (conn->host_client_id % 8));
	atomic_set(&conn->state, HECI_FILE_INITIALIZING);
	atomic_set(&file_ext->sm_state, 0);

	list_add_tail(&conn->link, &dev->conn_list);
	atomic_inc(&conn->refcnt);
	file_ext->conn = conn;
	list_add_tail(&file_ext->link, &dev->file_list);
	spin_unlock(&conn->file_lock);
	spin_unlock_bh(&dev->conn_lock);
	file->private_data = file_ext;

	return 0;
}

/**
 * heci_release - the release function
 *
 * @inode: pointer to inode structure
 * @file: pointer to file structure
 *
 * returns 0 on success, <0 on error
 */
static int heci_release(struct inode *inode, struct file *file)
{
	int rets = 0;
	struct heci_file_priv *file_ext = file->private_data;
	struct heci_connection *conn = NULL;
	struct iamt_heci_device *dev;

	DBG("entry\n");
	if (heci_device == NULL)
		return -ENODEV;

	if ((iminor(inode) != HECI_MINOR_NUMBER) || (file_ext == NULL))
		return -ENODEV;

	dev = pci_get_drvdata(heci_device);
	if (dev == NULL)
		return -ENODEV;

	conn = file_ext->conn;
	if (conn == NULL)
		return -ENODEV;

	spin_lock_bh(&conn->file_lock);
	if (conn->file == file)
		conn->canceled = 1;
	spin_unlock_bh(&conn->file_lock);

	_heci_file_cancel(dev, file, conn);

	if ((conn != &dev->iamthif_conn) &&
	    (atomic_cmpxchg(&conn->state, HECI_FILE_CONNECTED,
			    HECI_FILE_DISCONNECTING) == HECI_FILE_CONNECTED)) {
		DBG("disconnecting client host client = %d, ME client = %d\n",
			conn->host_client_id, conn->me_client_id);
		rets = heci_disconnect_host_client(dev, conn);
	}

	spin_lock_bh(&dev->conn_lock);
	spin_lock(&conn->file_lock);

	if (conn->file == file)
		conn->canceled = 1;

	if (conn != &dev->iamthif_conn)
		list_del_init(&conn->link);

	if (dev->open_handle_count > 0)
		dev->open_handle_count--;
	dev->heci_host_clients[file_ext->host_client_id / 8] &=
		~(1 << (file_ext->host_client_id % 8));
	file->private_data = NULL;
	list_del_init(&file_ext->link);
	file_ext->conn = NULL;

	spin_unlock(&conn->file_lock);
	spin_unlock_bh(&dev->conn_lock);

	if (conn != &dev->iamthif_conn)
		heci_free_heci_connection(conn);
	kfree(file_ext);

	return rets;
}

/**
 * heci_read - the read client message function.
 *
 * @file: pointer to file structure
 * @ubuf: pointer to user buffer
 * @length: size of user buffer
 * @offset: data offset in buffer
 *
 * returns >=0 read length on success, <0 on error
 */
static ssize_t heci_read(struct file *file, char __user *ubuf,
			 size_t length, loff_t *offset)
{
	int needfree = 0;
	__u8 rcomplete = 0;
	__u8 i;
	int rets = 0;
	int onlyput = 0;
	int if_num = iminor(file->f_dentry->d_inode);
	struct heci_file_priv *file_ext = file->private_data;
	struct heci_connection *conn = NULL;
	struct heci_cb_private *priv_cb = NULL;
	struct iamt_heci_device *dev;

	DBG("entry\n");
	if (heci_device == NULL)
		return -ENODEV;

	if ((if_num != HECI_MINOR_NUMBER) || (file_ext == NULL))
		return -ENODEV;

	dev = pci_get_drvdata(heci_device);
	if (dev == NULL)
		return -ENODEV;

	conn = file_ext->conn;
	if (conn == NULL)
		return -ENODEV;

	if ((length == 0) || (ubuf == NULL))
		return -EMSGSIZE;

	if (atomic_read(&dev->heci_state) != HECI_ENABLED)
		return -ENODEV;

	spin_lock_bh(&dev->mecl_lock);
	rets = heci_client_find(dev, conn, &i);
	if (rets) {
		BUG();
		DBG("client not found.\n");
		spin_unlock_bh(&dev->mecl_lock);
		return -ENODEV;
	}
	if (atomic_cmpxchg(&file_ext->sm_state, HECI_WD_STATE_INDEP, 0)
	    != HECI_WD_STATE_INDEP) {
		/* Do not allow to read watchdog client */
		if (memcmp(&heci_wd_guid,
			   &dev->me_clients[i].props.protocol_name,
			   sizeof(struct guid)) == 0) {
			spin_unlock_bh(&dev->mecl_lock);
			return -EBADF;
		}
	}
	spin_unlock_bh(&dev->mecl_lock);

	priv_cb = heci_find_read_entry(dev, file, &rcomplete);
	DBG("check read complete = %d\n", rcomplete);
	if (rcomplete == HECI_COMPLETED)
		goto read;
	if (priv_cb != NULL) {
		heci_put_io_cb(priv_cb);
		return 0;
	}

	if (conn == &dev->iamthif_conn)
		return -EINVAL;

	/* read not started */
	if (atomic_read(&conn->state) != HECI_FILE_CONNECTED) {
		DBG("host client = %d,  is not connected to ME client = %d",
		    conn->host_client_id,
		    conn->me_client_id);
		return -ENODEV;
	}

	if (*offset > 0) {
		/* Offset needs to be cleaned for contingous reads */
		DBG("read offset cleaning\n");
		*offset = 0;
		return 0;
	}

	DBG("starting read\n");
	if (conn->read_cb == NULL)
		priv_cb = heci_init_io_cb(conn, HECI_READ, file_ext,
						0, conn->max_msg_length);
	if (priv_cb == NULL)
		return -ENOMEM;

	file_ext->status = 0;
	atomic_set(&file_ext->r_state, HECI_READING);

	spin_lock_bh(&dev->cmdl_lock);
	list_add_tail(&priv_cb->cb_list, &dev->heci_cmd_list.cb_list);
	spin_unlock_bh(&dev->cmdl_lock);
	atomic_inc(&dev->recvd_wrr);
	wake_up_interruptible(&dev->wait_isr);

	if (file->f_flags & O_NONBLOCK)
		return -EAGAIN;

	DBG("waiting read\n");
	rets = wait_event_interruptible(file_ext->rx_wait,
		((atomic_read(&file_ext->r_state) == HECI_READ_COMPLETE) ||
		 (atomic_read(&conn->state) != HECI_FILE_CONNECTED)));
	if (rets == -ERESTARTSYS)
		return -EINTR;
	if (atomic_read(&conn->state) != HECI_FILE_CONNECTED)
		return -EBUSY;

	priv_cb = heci_find_read_entry(dev, file, &rcomplete);
	if (rcomplete != HECI_COMPLETED) {
		if (priv_cb != NULL)
			heci_put_io_cb(priv_cb);
		return 0;
	}

read:
	DBG("reading...\n");
	BUG_ON(priv_cb->conn != conn);
	rets = heci_read_raw(dev, file_ext, priv_cb, ubuf, length, offset,
		((conn == &dev->iamthif_conn) ? IAMTHIF_READ_TIMER : 0),
		&needfree);
	onlyput = heci_put_io_cb(priv_cb);
	if (needfree != 0) {
		if (onlyput == 0) {
			spin_lock_bh(&dev->rdcpl_lock);
			list_del_init(&priv_cb->cb_list);
			spin_unlock_bh(&dev->rdcpl_lock);
			heci_free_io_cb(priv_cb);
		}
		atomic_set(&file_ext->r_state, HECI_IDLE);
	}

	return rets;
}

/**
 * heci_check_special_wd_msg - checks for allowed special WD message
 *
 * @file_ext: private data of file object
 * @buf: pointer to message buffer
 * @length: message length
 */
static void heci_check_special_wd_msg(struct heci_file_priv *file_ext,
			 const char *buf, size_t length)
{
	if ((length == 4) &&
	    ((memcmp(heci_wd_state_independence_msg[0],	buf, 4) == 0) ||
	     (memcmp(heci_wd_state_independence_msg[1], buf, 4) == 0) ||
	     (memcmp(heci_wd_state_independence_msg[2], buf, 4) == 0)))
		atomic_set(&file_ext->sm_state, HECI_WD_STATE_INDEP);
	else
		atomic_set(&file_ext->sm_state, 0);
}

/**
 * heci_write - the write function.
 *
 * @file: pointer to file structure
 * @ubuf: pointer to user buffer
 * @length: size of user buffer
 * @offset: data offset in buffer
 *
 * returns >=0 write length on success, <0 on error
 */
static ssize_t heci_write(struct file *file, const char __user *ubuf,
			  size_t length, loff_t *offset)
{
	int rets = 0;
	__u8 i;
	int if_num = iminor(file->f_dentry->d_inode);
	struct heci_file_priv *file_ext = file->private_data;
	struct heci_connection *conn = NULL;
	struct heci_cb_private *priv_cb = NULL;
	struct iamt_heci_device *dev;

	DBG("entry\n");
	if (heci_device == NULL)
		return -ENODEV;

	if ((if_num != HECI_MINOR_NUMBER) || (file_ext == NULL))
		return -ENODEV;

	dev = pci_get_drvdata(heci_device);
	if (dev == NULL)
		return -ENODEV;

	conn = file_ext->conn;
	if (conn == NULL)
		return -ENODEV;

	*offset = 0;

	if (atomic_read(&dev->heci_state) != HECI_ENABLED)
		return -ENODEV;

	spin_lock_bh(&dev->rdcpl_lock);
	priv_cb = heci_find_list_file_entry(&dev->read_cmpl_list, file);
	if (priv_cb != NULL) {
		list_del_init(&priv_cb->cb_list);
		heci_free_io_cb(priv_cb);
		atomic_set(&file_ext->r_state, HECI_IDLE);
	}
	priv_cb = NULL;
	spin_unlock_bh(&dev->rdcpl_lock);

	if (atomic_read(&file_ext->w_state) == HECI_WRITING)
		return -EBUSY;

	if (conn == &dev->iamthif_conn) {
		priv_cb = heci_init_io_cb(conn,
				HECI_READ, file_ext, length, IAMTHIF_MTU);
	} else {
		priv_cb = heci_init_io_cb(conn,
				HECI_WRITE, file_ext, length, 0);
	}
	if (priv_cb == NULL)
		return -ENOMEM;
	DBG("length =%d\n", (int) length);

	if (copy_from_user(priv_cb->request_buffer.data,
		ubuf, length)) {
		rets = -EFAULT;
		goto fail;
	}

	heci_check_special_wd_msg(file_ext,
				priv_cb->request_buffer.data,
				length);

	if (atomic_read(&dev->heci_state) != HECI_ENABLED) {
		rets = -ENODEV;
		goto fail;
	}

	spin_lock_bh(&dev->mecl_lock);
	rets = heci_client_find(dev, conn, &i);
	if (rets) {
		BUG();
		spin_unlock_bh(&dev->mecl_lock);
		goto fail;
	}
	if ((length > dev->me_clients[i].props.max_msg_length)
		    || (length <= 0)) {
		spin_unlock_bh(&dev->mecl_lock);
		rets = -EMSGSIZE;
		goto fail;
	}
	spin_unlock_bh(&dev->mecl_lock);

	spin_lock_bh(&conn->file_lock);
	DBG("host client = %d, ME client = %d\n",
	    conn->host_client_id, conn->me_client_id);
	if (atomic_read(&conn->state) != HECI_FILE_CONNECTED) {
		spin_unlock_bh(&conn->file_lock);
		rets = -ENODEV;
		DBG("host client = %d,  is not connected to ME client = %d",
		    conn->host_client_id,
		    conn->me_client_id);
		goto fail;
	}
	if ((length > conn->max_msg_length) || (length <= 0)) {
		spin_unlock_bh(&conn->file_lock);
		rets = -EMSGSIZE;
		goto fail;
	}

	if (atomic_read(&file_ext->w_state) == HECI_WRITING) {
		spin_unlock_bh(&conn->file_lock);
		rets = -EBUSY;
		goto fail;
	}
	spin_unlock_bh(&conn->file_lock);
	atomic_set(&file_ext->w_state, HECI_WRITING);
	spin_lock_bh(&dev->cmdl_lock);
	if (conn == &dev->iamthif_conn)
		list_add_tail(&priv_cb->cb_list, &dev->pthi_cmd_list.cb_list);
	else
		list_add_tail(&priv_cb->cb_list, &dev->heci_cmd_list.cb_list);
	spin_unlock_bh(&dev->cmdl_lock);
	atomic_inc(&dev->recvd_wrr);
	DBG("write go\n");
	wake_up_interruptible(&dev->wait_isr);

	return length;

fail:
	heci_free_io_cb(priv_cb);
	return rets;

}

/**
 * heci_ioctl - the IOCTL function
 *
 * @file: pointer to file structure
 * @cmd: ioctl command
 * @data: pointer to heci message structure
 *
 * returns 0 on success, <0 on error
 */
static long heci_ioctl(struct file *file, unsigned int cmd, unsigned long data)
{
	int rets = 0;
	int if_num = iminor(file->f_dentry->d_inode);
	struct heci_file_priv *file_ext = file->private_data;
	struct heci_connection *conn = NULL;
	/* in user space */
	struct heci_message_data *u_msg = (struct heci_message_data *) data;
	struct heci_message_data k_msg;	/* all in kernel on the stack */
	struct iamt_heci_device *dev;

	if (!capable(CAP_SYS_ADMIN))
		return -EPERM;

	if (heci_device == NULL)
		return -ENODEV;

	if ((if_num != HECI_MINOR_NUMBER) || (file_ext == NULL))
		return -ENODEV;

	dev = pci_get_drvdata(heci_device);
	if (dev == NULL)
		return -ENODEV;

	conn = file_ext->conn;
	if (conn == NULL)
		return -ENODEV;

	if (atomic_read(&dev->heci_state) != HECI_ENABLED)
		return -ENODEV;

	/* first copy from user all data needed */
	if (copy_from_user(&k_msg, u_msg, sizeof(k_msg))) {
		DBG("first copy from user all data needed filled\n");
		return -EFAULT;
	}
	DBG("user message size is %d\n", k_msg.size);

	switch (cmd) {
	case IOCTL_HECI_GET_VERSION:
		DBG(": IOCTL_HECI_GET_VERSION\n");
		rets = heci_ioctl_get_version(dev, u_msg, k_msg);
		break;

	case IOCTL_HECI_CONNECT_CLIENT:
		DBG(": IOCTL_HECI_CONNECT_CLIENT.\n");
		rets = heci_ioctl_connect_client(dev, u_msg, k_msg, file_ext);
		break;

	case IOCTL_HECI_WD:
		DBG(": IOCTL_HECI_WD.\n");
		rets = heci_ioctl_wd(dev, k_msg);
		break;

	case IOCTL_HECI_BYPASS_WD:
		DBG(": IOCTL_HECI_BYPASS_WD.\n");
		rets = heci_ioctl_bypass_wd(dev, k_msg);
		break;

	default:
		rets = -EINVAL;
		break;
	}
	return rets;
}

/**
 * heci_poll - the poll function
 *
 * @file: pointer to file structure
 * @wait: pointer to poll_table
 *
 * returns poll mask
 */
static unsigned int heci_poll(struct file *file, poll_table *wait)
{
	int if_num = iminor(file->f_dentry->d_inode);
	unsigned int mask = 0;
	struct heci_file_priv *file_ext = file->private_data;
	struct heci_connection *conn = NULL;
	struct iamt_heci_device *dev;

	if (heci_device == NULL)
		return mask;

	if ((if_num != HECI_MINOR_NUMBER) || (file_ext == NULL))
		return mask;

	dev = pci_get_drvdata(heci_device);
	if (dev == NULL)
		return mask;

	conn = file_ext->conn;
	if (conn == NULL)
		return -ENODEV;

	if (atomic_read(&dev->heci_state) != HECI_ENABLED)
		return mask;

	if (conn == &dev->iamthif_conn) {
		poll_wait(file, &file_ext->rx_wait, wait);
	} else {
		poll_wait(file, &file_ext->tx_wait, wait);
	}

	if (atomic_read(&dev->heci_state) != HECI_ENABLED)
		return mask;

	if (conn == &dev->iamthif_conn) {
		if (atomic_read(&file_ext->r_state) == HECI_READ_COMPLETE)
			mask |= (POLLIN | POLLRDNORM);
	} else {
		if (atomic_read(&file_ext->w_state) == HECI_WRITE_COMPLETE)
			mask |= (POLLIN | POLLRDNORM);
	}

	return mask;
}

#ifdef CONFIG_PM
static int heci_suspend(struct pci_dev *pdev, pm_message_t state)
{
	struct iamt_heci_device *dev = pci_get_drvdata(pdev);
	int lck = 0;

	if (dev == NULL)
		return -ENODEV;

	spin_lock_bh(&dev->device_lock);
	if (dev->reinit_tsk != NULL) {
		set_tsk_thread_flag(dev->reinit_tsk, TIF_SIGPENDING);
		wake_up_process(dev->reinit_tsk);
	}
	spin_unlock_bh(&dev->device_lock);
	lck = down_trylock(&dev->reinit_sem); /* lock if can */

	_heci_stop_wd(dev);

	/* Set new heci state */
	if ((atomic_read(&dev->heci_state) == HECI_ENABLED) ||
	    (atomic_read(&dev->heci_state) == HECI_RECOVERING_FROM_RESET)) {
		atomic_set(&dev->heci_state, HECI_POWER_DOWN);
		spin_lock_bh(&dev->device_lock);
		heci_reset(dev, 0);
		spin_unlock_bh(&dev->device_lock);
	}

	pci_save_state(pdev);

	pci_disable_device(pdev);
	free_irq(pdev->irq, dev);

	pci_set_power_state(pdev, PCI_D3hot);

	if (lck)
		up(&dev->reinit_sem);
	return 0;
}

static int heci_resume(struct pci_dev *pdev)
{
	struct iamt_heci_device *dev;
	int err = 0;

	pci_set_power_state(pdev, PCI_D0);
	pci_restore_state(pdev);

	dev = pci_get_drvdata(pdev);
	if (dev == NULL)
		return -ENODEV;

	/* request and enable interrupt   */
	err = request_irq(pdev->irq, heci_isr_interrupt, IRQF_SHARED,
			heci_driver_name, dev);
	if (err) {
		printk(KERN_ERR "heci: Request_irq failure. irq = %d \n",
		       pdev->irq);
		return err;
	}

	spin_lock_bh(&dev->device_lock);
	atomic_set(&dev->heci_state, HECI_POWER_UP);
	heci_reset(dev, 1);

	/* Start watchdog if stopped in suspend */
	if (g_sus_wd_timeout != 0) {
		spin_lock(&dev->wd_conn.file_lock);
		dev->wd_timeout = g_sus_wd_timeout;

		memcpy(dev->wd_data, heci_start_wd_params, HECI_WD_PARAMS_SIZE);
		memcpy(dev->wd_data + HECI_WD_PARAMS_SIZE,
		       &dev->wd_timeout, sizeof(__u16));
		dev->wd_due_counter = 1;
		spin_unlock(&dev->wd_conn.file_lock);

		if (dev->wd_timeout)
			mod_timer(&dev->wd_timer, jiffies);

		g_sus_wd_timeout = 0;
	}
	spin_unlock_bh(&dev->device_lock);
	return err;
}
#endif

MODULE_AUTHOR("Intel Corporation");
MODULE_DESCRIPTION("Intel(R) Management Engine Interface");
MODULE_LICENSE("Dual BSD/GPL");
MODULE_VERSION(HECI_DRIVER_VERSION);
